@@ -17,7 +17,7 @@ use Bio::Gonzales::Util::File qw/open_on_demand slurpc expand_home/;
 
 use base 'Exporter';
 our ( @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-our $VERSION = '0.068'; # VERSION
+our $VERSION = '0.071'; # VERSION
 
 @EXPORT      = qw(mslurp mspew lslurp miterate lspew dict_slurp dict_spew xcsv_slurp miterate_hash);
 %EXPORT_TAGS = ();
@@ -26,8 +26,15 @@ my $COMMENT_RE = qr/^\s*#/;
 
 sub dict_slurp {
   my ( $src, $cc ) = @_;
+
+  croak "no config settings given"
+  unless($cc && ref $cc eq 'HASH');
+
+  $cc->{key_idx} //= $cc->{key_idcs};
+  $cc->{val_idx} //= $cc->{val_idcs};
+
   croak "you have not specified key_idx"
-    unless ( $cc && exists( $cc->{key_idx} ) );
+    unless ( exists( $cc->{key_idx} ) );
 
   $cc //= {};
   my %c = (
@@ -211,7 +218,6 @@ sub mslurp {
   }
 }
 
-
 sub xcsv_slurp {
   my ( $src, $c ) = @_;
 
@@ -338,7 +344,7 @@ sub mspew {
   my ( $dest, $m, $c ) = @_;
 
   confess "no matrix, you need to supply a matrix of the form [ [ 1,2,3 ], [ 4,5,6 ], ... ]"
-    unless ($m);
+    unless ($m && ref $m eq 'ARRAY');
 
   my $header    = $c->{header}    // $c->{ids};
   my $rownames  = $c->{row_names} // $c->{rownames};
