@@ -7,10 +7,11 @@ use strict;
 use Carp;
 use FindBin;
 use File::Spec;
+use Scalar::Util qw/readonly/;
 use Bio::Gonzales::Util::File qw/slurpc/;
 use Bio::Gonzales::Util::Cerial;
 use Bio::Gonzales::Util::Development::File;
-use Data::Visitor::Callback;
+use Data::Rmap qw/rmap_scalar rmap_to :types/;
 use Bio::Gonzales::Util::Log;
 use Data::Printer {
   indent         => 2,
@@ -23,7 +24,7 @@ use POSIX;
 
 use 5.010;
 
-our $VERSION = '0.068'; # VERSION
+our $VERSION = '0.071'; # VERSION
 
 has '_config_key_cache' => ( is => 'rw', default => sub { {} } );
 has '_nfi_cache'        => ( is => 'rw', default => sub { {} } );
@@ -33,12 +34,12 @@ has 'config'           => ( is => 'rw', lazy_build => 1 );
 has 'merge_av_config'  => ( is => 'rw', default    => 1 );
 has 'log'              => ( is => 'rw', builder    => '_build_log' );
 has 'config_file'      => ( is => 'rw', default    => 'gonz.conf.yml' );
-has 'analysis_name' => (is => 'rw', lazy_build => 1);
+has 'analysis_name'    => ( is => 'rw', lazy_build => 1 );
 
 sub _build_analysis_name {
   my ($self) = @_;
 
-  return (File::Spec->splitdir(File::Spec->rel2abs('.')))[-1]
+  return ( File::Spec->splitdir( File::Spec->rel2abs('.') ) )[-1];
 }
 
 sub _build_analysis_version {
@@ -66,19 +67,22 @@ sub _build__substitute_conf {
     data    => sub { return $self->path_to('data') },
   );
 
-  return Data::Visitor::Callback->new(
-    plain_value => sub {
-      return unless defined $_;
-      $_ =~ s{ ^ ~ ( [^/]* ) }
+  my $subsre = join "|", keys %subs;
+
+  return sub {
+    return unless defined $_[0];
+    # boolean values in YAML::XS are readonly. Take care of this.
+    return $_[0] if(readonly($_[0]));
+
+      $_[0] =~ s{ ^ ~ ( [^/]* ) }
             { $1
                 ? (getpwnam($1))[7]
                 : ( $ENV{HOME} || (getpwuid($>))[7] )
             }ex;
 
-      my $subsre = join "|", keys %subs;
-      s{__($subsre)(?:\((.+?)\))?__}{ $subs{ $1 }->( $2 ? split( /,/, $2 ) : () ) }eg;
+      $_[0] =~ s{__($subsre)(?:\((.+?)\))?__}{ $subs{ $1 }->( $2 ? split( /,/, $2 ) : () ) }eg;
+    return $_[0];
     }
-  );
 }
 
 sub _build_log {
@@ -96,14 +100,16 @@ sub _build_config {
 
   my $conf;
   my $conf_f = $self->config_file;
+  my $sub    = $self->_substitute_conf;
+
   if ( -f $conf_f ) {
     $conf = yslurp($conf_f);
-  $conf //= {};
+    $conf //= {};
 
     confess "configuration file >> $conf_f << is not a hash/dictionary structure"
       if ( ref $conf ne 'HASH' );
     $self->log->info("reading >> $conf_f <<");
-    $self->_substitute_conf->visit($conf);
+    rmap_to { $sub->($_) } VALUE, $conf;
   }
 
   my $av_conf_f = join( ".", $self->analysis_version, "conf", "yml" );
@@ -114,7 +120,7 @@ sub _build_config {
       if ( ref $av_conf ne 'HASH' );
 
     $self->log->info("reading >> $av_conf_f <<");
-    $self->_substitute_conf->visit($av_conf);
+    rmap_to { $sub->($_) } VALUE, $conf;
 
     $conf = { %$conf, %$av_conf };
   }
@@ -201,11 +207,11 @@ sub conf {
   }
   if (@keys) {
     my $k = join( " ", @keys );
-    $self->log->info( "(gonzconf) > " . $k . " <", p($data) )
+    $self->log->info( "(gonzconf) > " . $k . " <", np($data) )
       unless ( $self->_config_key_cache->{ '_' . $k }++ );
 
   } else {
-    $self->log->info( "(gonzconf) dump", p($data) )
+    $self->log->info( "(gonzconf) dump", np($data) )
       unless ( $self->_config_key_cache->{'_'}++ );
   }
   return $data;
